local addonName, addon = ...

if addon.platform then return end

local default = addon.defaultSettings
default.anchorPoint			= 'CENTER'
default.anchorOffsetX		= 0
default.anchorOffsetY		= 0
default.borderSpacing		= 3
default.buttonSize			= 20
default.frameBackgroundA	= 0.69
default.frameBackgroundB	= 0.1
default.frameBackgroundG	= 0.05
default.frameBackgroundR	= 0.05
default.frameBorderA		= 1
default.frameBorderB		= 1
default.frameBorderG		= 1
default.frameBorderR		= 1
default.frameLevel			= 1
default.frameLock			= false
default.frameStrata			= 'HIGH'

local frame, order = addon.frame, addon.order

local LDA = LibStub('LibDisplayAssist-1.4')
local GetRelOffset = LDA.GetRelOffset
local SetRelOffset = LDA.SetRelOffset

--[[----------------------------------------------------------------------------
Support
------------------------------------------------------------------------------]]
function addon.SaveOffset()
	local settings = addon.db.profile
	settings.anchorOffsetX, settings.anchorOffsetY = GetRelOffset(frame, settings.anchorPoint, UIParent, settings.anchorPoint, true)
end

function addon.SetOffset()
	local settings = addon.db.profile
	frame:ClearAllPoints()
	SetRelOffset(frame, settings.anchorPoint, UIParent, settings.anchorPoint, settings.anchorOffsetX, settings.anchorOffsetY, true)
	addon.UpdateAlertAnchoring()
end

--[[----------------------------------------------------------------------------
Script abstraction
------------------------------------------------------------------------------]]
function addon.OnDragStart()
	if addon.db.profile.frameLock then return end
	addon.isMoving = true
	frame:StartMoving()
	GameTooltip:Hide()
	addon.HideAlerts()
end

function addon.OnDragStop()
	if not addon.isMoving then return end
	addon.isMoving = nil
	frame:StopMovingOrSizing()
	addon.SaveOffset()
	addon.SetOffset()
end

addon.OnEnter = addon.DoNothing
addon.OnLeave = addon.DoNothing

--[[----------------------------------------------------------------------------
Addon methods
------------------------------------------------------------------------------]]
function addon:ApplyPluginSettings()
	local settings = self.db.profile
	addon.SetOffset()
	frame:SetFrameStrata(settings.frameStrata)
	frame:SetFrameLevel(settings.frameLevel)
	frame:SetBackdropColor(settings.frameBackgroundR, settings.frameBackgroundG, settings.frameBackgroundB, settings.frameBackgroundA)
	frame:SetBackdropBorderColor(settings.frameBorderR, settings.frameBorderG, settings.frameBorderB, settings.frameBorderA)
	self.UpdatePlugin()
end

function addon:IsTooltipAllowed()
	return SecureCmdOptionParse(self.db.profile.tooltip) ~= "Hide"
end

function addon.UpdatePlugin()
	local settings, button = addon.db.profile
	local borderSpacing, buttonSize, spacing = settings.borderSpacing, settings.buttonSize, settings.spacing
	local baseOffset, offsetAdjust = buttonSize + spacing, borderSpacing + 3
	for index = 1, #order do
		button = order[index]
		button:SetSize(buttonSize, buttonSize)
		button:ClearAllPoints()
		button:SetPoint('LEFT', frame, baseOffset * (index - 1) + offsetAdjust, 0)
	end
	frame:SetSize(baseOffset * #order - spacing + offsetAdjust + offsetAdjust, buttonSize + offsetAdjust + offsetAdjust)
end

--[[----------------------------------------------------------------------------
Initialize
------------------------------------------------------------------------------]]
frame:SetBackdrop({
	bgFile = [[Interface\BUTTONS\WHITE8X8]],
	edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]], edgeSize = 16,
	tileSize = 16, tile = true,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
})
frame:SetClampedToScreen(true)
frame:SetClampRectInsets(3, -3, -3, 3)
frame:Show()

LDA.RegisterCallback(addon, 'DISPLAY_SIZE_CHANGED', addon.SetOffset)
LDA.RegisterCallback(addon, 'PLAYER_LOGOUT', addon.SaveOffset)
